function [DWVolume S0Volume b schemeDW spx spy spz maskNoBackground] = DW_data(schemeFileName,fileListName)
%DW_data      read the DW, B0, scheme and DBF data from nifti files
%
%   [DWVolume S0Volume b scheme spx spy spz maskNoBackground] = DW_data(schemeFileName,fileListName) 
%   Given the files SCHEMEFILENAME with the aquisition scheme (see below for file format description) and FILELISTNAME with the list of 
%   DW and S0 files the function reads DW files and returns matlab volumes. The function removes the background (MASKNOBACKGROUND)
%   based on the value of addition of the DW signals per voxel, valid voxels are those that the sum of their DW signasl are bigger than 
%   internal parameter thSumDW (~ 6000).
%
%
%   Files format
%
%   SCHEMEFILENAME with M= #DW+#B0 normalized encoding directions must be a text file with values:
%   bValue
%   X_direction1 Y_direction1 Z_Direction1
%   X_direction2 Y_direction2 Z_Direction2
%   ...
%   X_directionM Y_directionM Z_DirectionM
%       
%       - B0 images are denoted by  a row of zeros : 0 0 0
%       - it is possible to have several B0 entries in the scheme in any
%       arbitrary position.
%       - The order of this scheme file must correspond with the order of
%       files in FILELISTNAME below
%
%
%
%   FILELISTNAME is text file containing the path and the list of M = #DW+#B0 files:
%   path_of_files
%   fileName1
%   fileName2
%   ...
%   fileNameM
%
%       - The order of this list of files must correspond with the order of
%       the scheme file SCHEMEFILENAME above.
%
%   - It is possible also to use repetitions of DW signals, thus  
%   all subsets of single DW acquisition must have the same order, 
%   for example, these are the entries for 3 repetitions of the same 4 
%   diffusion encoding orientations in the SCHEMEFILENAME file:
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$


% %%%%%%%%%%%%%%%%%%% Starting

thSumDW = 6000; % change this for different scale in DW images!!

%read scheme file
fp = fopen(schemeFileName,'r');
scheme = fscanf(fp,'%f');
fclose(fp);


b = scheme(1);
%sort for Nx3 format
scheme = scheme(2:size(scheme,1));
scheme = reshape(scheme,3,size(scheme,1)/3)';

%read File list B0+DW
fileList = textread(fileListName,'%s');
%get path
pathNIIFiles =  char(fileList(1));
%get files
fileList = fileList(2:size(fileList,1));

%get the B0 images filenames
fileListB0 = fileList(sum(abs(scheme),2) <= 1e-1);

%get the DW images filenames
idxDW = ones(size(fileList,1),1); idxDW(find(sum(abs(scheme),2) <= 1e-1))=0;
fileListDW = fileList(idxDW==1);

schemeB0 = scheme(sum(abs(scheme),2)  <= 1e-1,:);

% keep scheme only with DW directions
schemeDW = scheme(sum(abs(scheme),2)  > 1e-1,:);

if size(fileListDW,1) ~= size(schemeDW,1) || size(fileListB0,1) ~= size(schemeB0,1) %Mod takes into account repetitions of same diffusion direction
    error('information (# of files) in scheme and file names does not match!!')
end

% read B0s
nii = nifti([pathNIIFiles char(fileListB0(1))]);
S0Volume = nii2matlab(nii.dat(:,:,:,:));
for i=2:size(fileListB0,1)
    nii = nifti([pathNIIFiles char(fileListB0(i))]);
    S0Volume = S0Volume + nii2matlab(nii.dat(:,:,:,:));
end
S0Volume = S0Volume / size(fileListB0,1); % divide by number of repetitions
[ny,nx,nz] = size(S0Volume);

% get voxel size
spx = abs(nii.mat(1,1));
spy = abs(nii.mat(2,2));
spz = abs(nii.mat(3,3));


%Check number of gradients by taking into account repetitions (averaging)
nGradients = size(schemeDW,1) / max(sum( abs(schemeDW*schemeDW')  > 0.99,1));
nRept = size(schemeDW,1)/nGradients;
for i=2:nRept
    if max(abs(schemeDW(1:nGradients,:) - schemeDW((i-1)*nGradients+(1:nGradients),:))) > 1e-3
        error('there are repetitions of DW orientations but the entries are not the same or are not sorted!!')
    end
end


%read DWs
DWVolume = zeros(ny,nx,nz,nGradients);
for i=0:size(fileListDW,1)-1
    nii = nifti([pathNIIFiles char(fileListDW(i+1))]);
    % average, if that is the case of repetitions
    DWVolume(:,:,:,mod(i,nGradients)+1) = DWVolume(:,:,:,mod(i,nGradients)+1) + nii2matlab(nii.dat(:,:,:,:));
end

DWVolume = DWVolume / nRept; %in case of repetitions

%compute mask based on background is darker
maskNoBackground = maskHead(double(sum(DWVolume,4)>thSumDW));

%the final scheme with only the original orientations
schemeDW = schemeDW(1:nGradients,:);


fprintf('\nVolumes read...\n');


