function fit2RankTensor(schemeFileName,fileListName,opath,nameVolume,rangx,rangy,rangz)
%fit2RankTensor      Fit a 2-rank diffusion data to a volume of DW images
%
%   fit2RankTensor(schemeFileName,fileListName,opath,nameVolume,rangx,rangy,rangz)
%   Given the files SCHEMEFILENAME with the aquisition scheme (see below for file format description), FILELISTNAME with the list of 
%   DW and S0 files, the function writes the result for ROI defined by ranges RANGX, RANGY, RANGZ  (use -1 if you want to process 
%   the whole image dimension) in the directory defined by OPATH, with the generic 
%   name for all files as NAMEVOLUME.
%
%
%   Files format
%
%   SCHEMEFILENAME with M= #DW+#B0 normalized encoding directions must be a text file with values:
%   bValue
%   X_direction1 Y_direction1 Z_Direction1
%   X_direction2 Y_direction2 Z_Direction2
%   ...
%   X_directionM Y_directionM Z_DirectionM
%       
%       - B0 images are denoted by  a row of zeros : 0 0 0
%       - it is possible to have several B0 entries in the scheme in any
%       arbitrary position.
%       - The order of this scheme file must correspond with the order of
%       files in FILELISTNAME below
%
%
%
%   FILELISTNAME is text file containing the path and the list of M = #DW+#B0 files:
%   path_of_files
%   fileName1
%   fileName2
%   ...
%   fileNameM
%
%       - The order of this list of files must correspond with the order of
%       the scheme file SCHEMEFILENAME above.
%
%   - It is possible also to use repetitions of DW signals, thus  
%   all subsets of single DW acquisition must have the same order, 
%   for example, these are the entries for 3 repetitions of the same 4 
%   diffusion encoding orientations in the SCHEMEFILENAME file:
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%
%
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$

[DW, s0 b orientations spx spy spz mask] = DW_data(schemeFileName,fileListName);
[ny,nx,nz,nv] = size(DW);

%recover righ range
rangx = giveRange(rangx,1,nx);
rangy = giveRange(rangy,1,ny);
rangz = giveRange(rangz,1,nz);


s0 = s0(rangy,rangx,rangz);
mask = mask(rangy,rangx,rangz);
DW = DW(rangy,rangx,rangz,:);

sRangeDW = ['_numDW' int2str(nv)];


nVoxels = sum(sum(sum(mask)));
fprintf('\nfit2RankTensor Number of voxels: %d',nVoxels);

[ny,nx,nz,nv] = size(DW);


rank =2;
sRank = ['_rank' int2str(rank)];

NLEC = nv;


[nCoef CoeffDeff nxsyszs mu matHRT fac_m_i_b_mu T] = preComputeHRTenData(rank,NLEC,orientations,b);


%Start!
MDs = zeros(size(s0));
HRT_coeffs = zeros(ny,nx,nz,nCoef);
PDD = zeros(ny,nx,nz,3);
lambdas = zeros(ny,nx,nz,3);
faRC = zeros(ny,nx,nz);


fprintf('\nHRT Fitting. Now Processing:');    

warning off MATLAB:nearlySingularMatrix
for pz=1:nz
    fprintf('\nslice %d',pz);    
    for px=1:nx
        for py=1:ny
                        
            if ~mask(py,px,pz)
                continue;
            end
            
            [Si,S0r] = DWMRI_selectPosVol(DW, s0,py,px,pz);
            if min(Si)<=0 || S0r<=0
                continue;
            end

            
            [GA HRT_coeff MD gentr V] = computeHighRankTensor(Si,S0r,NLEC,nxsyszs,mu,matHRT,fac_m_i_b_mu,nCoef,T);

            MDs(py,px,pz) = MD;
            HRT_coeffs(py,px,pz,:) = HRT_coeff;
            
            Tad = [HRT_coeffs(py,px,pz,1) HRT_coeffs(py,px,pz,4) HRT_coeffs(py,px,pz,5);...
                HRT_coeffs(py,px,pz,4) HRT_coeffs(py,px,pz,2) HRT_coeffs(py,px,pz,6);...
                HRT_coeffs(py,px,pz,5) HRT_coeffs(py,px,pz,6) HRT_coeffs(py,px,pz,3)];
            %compute tensor coeffs
            [Vad,Dad] = eig(Tad);
            if min(diag(Dad)) < 0
                [Dad, HRT_coeff]=fromNeg2PosDefinite(Vad,Dad);
                HRT_coeffs(py,px,pz,:) = HRT_coeff;            
            end

            [valad idxad] = max(diag(Dad));

            PDD(py,px,pz,:)= Vad(:,idxad);
            lambdas(py,px,pz,:)= sort(diag(Dad));

            deno = sqrt(Dad(1,1)^2+Dad(2,2)^2+Dad(3,3)^2);
            faRC(py,px,pz) = (sqrt(3)/sqrt(2))*(sqrt( (Dad(1,1)-MD)^2+(Dad(2,2)-MD)^2+(Dad(3,3)-MD)^2 ) /deno);
                       
        end
    end
end

%just in case
faRC(faRC > 1) = 1;

% save a Mask according FA>0.20
maskFA = double(faRC>0.20);
se = strel('diamond',1); % we applied a morphological opening
maskFA = imdilate(maskFA,se);
maskFA = imerode(maskFA,se);
maskFA = maskFA .* mask;
    
%eliminate liquid or gray matter voxels (+- 3 sigma distribution of mean diffusion)
[maxVMD, meanVMD, minVMD, stdVMD] = statsNDVolume(MDs(:,:,:),maskFA);
bigMD = (double((MDs > (meanVMD + 3.0*stdVMD))) + double(MDs < (meanVMD - 3.0*stdVMD))).*maskFA;
maskAfterBigMD = maskFA - bigMD;

% Eliminate liquid regions from volumes
HRT_coeffs(:,:,:,1) = HRT_coeffs(:,:,:,1).*maskAfterBigMD;
HRT_coeffs(:,:,:,2) = HRT_coeffs(:,:,:,2).*maskAfterBigMD;
HRT_coeffs(:,:,:,3) = HRT_coeffs(:,:,:,3).*maskAfterBigMD;
HRT_coeffs(:,:,:,4) = HRT_coeffs(:,:,:,4).*maskAfterBigMD;
HRT_coeffs(:,:,:,5) = HRT_coeffs(:,:,:,5).*maskAfterBigMD;
HRT_coeffs(:,:,:,6) = HRT_coeffs(:,:,:,6).*maskAfterBigMD;

PDD(:,:,:,1) = PDD(:,:,:,1).*maskAfterBigMD;
PDD(:,:,:,2) = PDD(:,:,:,2).*maskAfterBigMD;
PDD(:,:,:,3) = PDD(:,:,:,3).*maskAfterBigMD;

lambdas(:,:,:,1) = lambdas(:,:,:,1).*maskAfterBigMD;
lambdas(:,:,:,2) = lambdas(:,:,:,2).*maskAfterBigMD;
lambdas(:,:,:,3) = lambdas(:,:,:,3).*maskAfterBigMD;


%Compute linear spherical and planar coefficients
LAPS = linearPlanarSphericalCoeffs(lambdas);
maskLinearMax = double(((LAPS(:,:,:,1)>LAPS(:,:,:,2)) & (LAPS(:,:,:,1)>LAPS(:,:,:,3))));

%compute an estimate for the lambda for the basis
[dummy1, meanL1, dummy2, dummy3] = statsNDVolume(squeeze(lambdas(:,:,:,3)),maskLinearMax);
[dummy1, meanL2, dummy2, dummy3] = statsNDVolume(squeeze(lambdas(:,:,:,2)),maskLinearMax);
%[dummy1, meanV, dummy2, dummy3] = statsNDVolume(squeeze(lambdas(:,:,:,1)),maskLinearMax);

% write DT coeffs in DTITK viewer format    
DTCoeffs = zeros(ny,nx,nz,1,6);
DTCoeffs(:,:,:,1,1) = HRT_coeffs(:,:,:,1);
DTCoeffs(:,:,:,1,2) = HRT_coeffs(:,:,:,4);
DTCoeffs(:,:,:,1,3) = HRT_coeffs(:,:,:,2);
DTCoeffs(:,:,:,1,4) = HRT_coeffs(:,:,:,5);
DTCoeffs(:,:,:,1,5) = HRT_coeffs(:,:,:,6);
DTCoeffs(:,:,:,1,6) = HRT_coeffs(:,:,:,3);


%Save output volumes
saveMatlab2Nifti([opath,'DT_FA_' nameVolume sRangeDW sRank '.nii'],faRC,'NONE', spx, spy, spz);
saveMatlab2Nifti([opath,'DT_MD_' nameVolume sRangeDW sRank '.nii'],MDs,'NONE', spx, spy, spz);
saveMatlab2Nifti([opath,'DT_PDDs_' nameVolume sRangeDW sRank '.nii'],PDD,'NONE', spx, spy, spz);
saveMatlab2Nifti([opath,'DT_maskFA_020_' nameVolume sRangeDW sRank '.nii'],maskFA,'NONE', spx, spy, spz);
saveMatlab2Nifti([opath,'DT_Coeffs_DTITKViewer_' nameVolume sRangeDW sRank '.nii'],DTCoeffs,'MATRIX_SYM', spx, spy, spz);

%write the PDD and Transversal Diffusion for DBF
fp = fopen([opath,'DBF_params_' nameVolume sRangeDW sRank '.dat'],'w');
fprintf(fp,'%.15f \n%.15f',meanL1,meanL2);
fclose(fp);


fprintf('\nDT fitting process finished\n');
