function fitDBFVolume(schemeFileName,fileListName,maskFileName,DBFDirectionsFileName,DBFParamsFileName,opath,nameVolume,rangx,rangy,rangz)
%fitDBFVolume      Main function for fitting DBF model
%
%   fitDBFVolume(schemeFileName,fileListName,maskFileName,DBFDirectionsFileName,DBFParamsFileName,opath,nameVolume,rangx,rangy,rangz) 
%   Given the files SCHEMEFILENAME with the aquisition scheme (see below for file format description), FILELISTNAME with the list of 
%   DW and S0 files, activity mask filename MASKFILENAME, basis directions for the DBF model DBFDIRECTIONSFILENAME, and DBFPARAMSFILENAME, 
%   the function writes the DBF result for ROI defined by ranges RANGX, RANGY, RANGZ  (use -1 if you want to process the whole image 
%   dimension) in the directory defined by OPATH, with the generic 
%   name for all files as NAMEVOLUME.
%
%
%   Files format
%
%   SCHEMEFILENAME with M= #DW+#B0 normalized encoding directions must be a text file with values:
%   bValue
%   X_direction1 Y_direction1 Z_Direction1
%   X_direction2 Y_direction2 Z_Direction2
%   ...
%   X_directionM Y_directionM Z_DirectionM
%       
%       - B0 images are denoted by  a row of zeros : 0 0 0
%       - it is possible to have several B0 entries in the scheme in any
%       arbitrary position.
%       - The order of this scheme file must correspond with the order of
%       files in FILELISTNAME below
%
%
%
%   FILELISTNAME is text file containing the path and the list of M = #DW+#B0 files:
%   path_of_files
%   fileName1
%   fileName2
%   ...
%   fileNameM
%
%       - The order of this list of files must correspond with the order of
%       the scheme file SCHEMEFILENAME above.
%
%   - It is possible also to use repetitions of DW signals, thus  
%   all subsets of single DW acquisition must have the same order, 
%   for example, these are the entries for 3 repetitions of the same 4 
%   diffusion encoding orientations in the SCHEMEFILENAME file:
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%    -0.9999   -0.0004    0.0149
%    -0.0034    0.9794   -0.2018
%    -0.0411    0.4815   -0.8755
%     0.5895   -0.8013   -0.1023
%
%   MASKFILENAME is the name of 3D nifti (.hdr+.img or .nii) file with the binary mask of voxels to be processed
%
%
%   DBFDIRECTIONSFILENAME is a text file with the N normalized basis directions for the DBF model:
%   X_direction1 Y_direction1 Z_Direction1
%   X_direction2 Y_direction2 Z_Direction2
%   ...
%   X_directionN Y_directionN Z_DirectionN
%       
%
%   DBFPARAMSFILENAME is a text file containing the longitudinal and transversal diffusion for the DBF model (function fit2RankTensor generate this file):
%   longitudinal_diffusion
%   transversal_diffusion
%
%
%
%   Alonso Ramirez.  
%   $Date: 2008/10/21$



[DW, s0 b orientations spx spy spz mask] = DW_data(schemeFileName,fileListName);
[ny,nx,nz,nv] = size(DW);

BasisDir = load(DBFDirectionsFileName);
par = load(DBFParamsFileName); DPDD = par(1); DTra = par(2);

nii = nifti(maskFileName);
maskFA = nii2matlab(nii.dat(:,:,:,:));


DBFs = createDBF( DPDD, DTra, BasisDir, orientations , b);


%recover range of image
rangx = giveRange(rangx,1,nx);
rangy = giveRange(rangy,1,ny);
rangz = giveRange(rangz,1,nz);


s0 = s0(rangy,rangx,rangz);
%mask = mask(rangy,rangx,rangz);
DW = DW(rangy,rangx,rangz,:);
mask = maskFA;


sRangeDW = int2str(nv);



nVoxels = sum(sum(sum(mask)));
fprintf('\nNumber of voxels: %d',nVoxels);


LBP = fitDBF(s0,DW,mask,DBFs,'NNLS');

%just in case you want to save the preliminar discrete results
%saveMatlab2Nifti([ opath 'alphas_' nameVolume '_' method '_DWN_' sRangeDW '.nii'],LBP,'NONE',spx,spy,spz);


% get a continuous version
numberBasisDNeig = 6;
[basisDNeigh, dotsNeig] = getBasisDirectionsNeighborhoods(numberBasisDNeig,BasisDir);
threshold = 1e-2;


[fNDiff,fPDD,fAmountDiff,fSizeClusters] = computeContiniousField(LBP,BasisDir,basisDNeigh,threshold,mask);

plotContinuosField(fPDD,fAmountDiff,mask,fNDiff);

continuous2NIFTI(fNDiff,fPDD,fAmountDiff,mask,opath,[nameVolume '_DWN_' sRangeDW ],DPDD,DTra,spx, spy, spz);


%saving fa for DTITK viewer
saveMatlab2Nifti([ opath 'B0_' nameVolume '_DWN_' sRangeDW '.nii'],s0,'NONE',spx,spy,spz);


fprintf('\nDBF Process finished\n');
